@@ -532,7 +532,7 @@ public class QtBluetoothLE {
         }
 
         try {
-            // BluetoothDevice.,connectGatt(Context, boolean, BluetoothGattCallback, int) was
+            // BluetoothDevice.connectGatt(Context, boolean, BluetoothGattCallback, int) was
             // officially introduced by Android API v23. Earlier Android versions have a private
             // implementation already though. Let's check at runtime and use it if possible.
             //
@@ -1126,7 +1126,7 @@ public class QtBluetoothLE {
         return true;
     }
 
-    // Called by TimeoutRunnable if the current I/O job timeoutd out.
+    // Called by TimeoutRunnable if the current I/O job timed out.
     // By the time we reach this point the handleForTimeout counter has already been reset
     // and the regular responses will be blocked off.
     private void interruptCurrentIO(int handle)
@@ -1141,16 +1141,24 @@ public class QtBluetoothLE {
         if (handle == HANDLE_FOR_MTU_EXCHANGE)
             return;
 
-        GattEntry entry = entries.get(handle);
-        if (entry == null)
-            return;
-        if (entry.valueKnown)
-            return;
-        entry.valueKnown = true;
+        try {
+            synchronized (this) {
 
-        GattEntry serviceEntry = entries.get(entry.associatedServiceHandle);
-        if (serviceEntry != null && serviceEntry.endHandle == handle)
-            finishCurrentServiceDiscovery(entry.associatedServiceHandle);
+                GattEntry entry = entries.get(handle);
+                if (entry == null)
+                    return;
+                if (entry.valueKnown)
+                    return;
+                entry.valueKnown = true;
+
+                GattEntry serviceEntry = entries.get(entry.associatedServiceHandle);
+                if (serviceEntry != null && serviceEntry.endHandle == handle)
+                    finishCurrentServiceDiscovery(entry.associatedServiceHandle);
+            }
+        } catch (IndexOutOfBoundsException outOfBounds) {
+            Log.w(TAG, "interruptCurrentIO(): Unknown gatt entry, index: "
+                    + handle + " size: " + entries.size());
+        }
     }
 
     /*
@@ -1271,9 +1279,16 @@ public class QtBluetoothLE {
                     }
 
                     // last entry of current discovery run?
-                    GattEntry serviceEntry = entries.get(entry.associatedServiceHandle);
-                    if (serviceEntry.endHandle == handle)
-                        finishCurrentServiceDiscovery(entry.associatedServiceHandle);
+                    synchronized (this) {
+                        try {
+                            GattEntry serviceEntry = entries.get(entry.associatedServiceHandle);
+                            if (serviceEntry.endHandle == handle)
+                                finishCurrentServiceDiscovery(entry.associatedServiceHandle);
+                        } catch (IndexOutOfBoundsException outOfBounds) {
+                            Log.w(TAG, "performNextIO(): Unknown service for entry, index: "
+                                            + entry.associatedServiceHandle + " size: " + entries.size());
+                        }
+                    }
                 } else {
                     int errorCode = 0;
 
